==> ciao-4.6-src-core.tar.gz: ciao-4.6/src/lib/datamodel/ftkernel/ftlib.c <==
/*                                                                
**  Copyright (C) 2002-2003,2007  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   ftlib.c                                                     */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmilib.h"
#include "ftkernelP.h"


/***************************************************************/
/*                                                             */
/* Function:   ftp_null_keyword_compose                        */
/*                                                             */
/* Description:                                                */
/*            Bad things happen to CFITS if you try to write   */
/*            a TNULL keyword to a non-integer column.         */
/*                                                             */
/*  Input parameters:                                          */
/*             const  dmDataType  type                         */
/*             const  long        colno                        */
/*                    char       *buf                          */
/*                    int         is_image                     */
/*                                                             */
/*  Output parameters:                                         */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*                 nothing                                     */
/*                                                             */
/***************************************************************/
void ftp_null_keyword_compose( const dmDataType type, const long colno,
                               char* buf, int is_image )
{
  if ( is_image )
  {
    cs_copy( "BLANK", buf );
    return;
  }

  switch( type )
  {
    case DT_INT4:
    case DT_INT2:
    case DT_UINT2:
    case DT_UINT4:
    case DT_UINT1:
    case DT_BIT:
      sprintf( buf, "TNULL%ld", colno );
      break;
    case DT_CHAR:
    case DT_REAL4:
    case DT_REAL8:
    default:
      sprintf( buf, "TDNULL%ld", colno );
      break;
  }
}


/***************************************************************/
/*                                                             */
/* Function:   ftp_block_type                                  */
/*                                                             */
/* Description:                                                */
/*            Convert FITSIO block type to ASCDM block type    */
/*                                                             */
/*  Input parameters:                                          */
/*                    int    ft_hdutype                        */
/*                                                             */
/*  Output parameters:                                         */
/*                    none                                     */
/*                                                             */
/*  Returns:                                                   */
/*                 dmBlockType  type                           */
/*                                                             */
/***************************************************************/
dmBlockType ftp_block_type( int ft_hdutype )
{
  switch( ft_hdutype )
  {
    case IMAGE_HDU:
      return dmIMAGE;

    case ASCII_TBL:
      return dmTABLE;

    case BINARY_TBL:
      return dmTABLE;

    default:
      break;
  }

  return dmUNKNOWNBLOCK;
}



/***************************************************************/
/*                                                             */
/* Function:   ftp_hdutype_name                                */
/*                                                             */
/* Description:                                                */
/*            Map CFITSIO HDU types to names                   */
/*                                                             */
/*  Input parameters:                                          */
/*               const int   type                              */
/*                     char *name                              */
/*                                                             */
/*  Output parameters:                                         */
/*                    none                                     */
/*                                                             */
/*  Returns:                                                   */
/*                 nothing                                     */
/*                                                             */
/***************************************************************/
void ftp_hdutype_name( const int type, char* name )
{
  char *names[] = { "IMAGE", "TABLE", "BINTABLE" };

  if ( type >= 0 && type <= 2 )
    strcpy( name, names[type] );
  else
    strcpy( name, "UNKNOWN" );
}



/***************************************************************/
/*                                                             */
/* Function:   ftp_bitpix                                      */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                 dmDataType  type                            */
/*                                                             */
/*  Output parameters:                                         */
/*                 double     *bscale                          */
/*                 double     *bzero                           */
/*                                                             */
/*  Returns:                                                   */
/*                 int  bitpix                                 */
/*                                                             */
/***************************************************************/
int ftp_bitpix( dmDataType type, double* bscale, double* bzero )
{
  *bscale = 1.0;
  *bzero  = 0.0;

  switch( type )
  {
    case DT_REAL8:
      return DOUBLE_IMG;

    case DT_REAL4:
      return FLOAT_IMG;

    case DT_INT2:
      return SHORT_IMG;

    case DT_INT4:
      return LONG_IMG;

    case DT_UINT1:
      return BYTE_IMG;

    case DT_UINT2: 
      *bscale = 1.0;
      *bzero  = USHORT_TZERO;
      return SHORT_IMG;  /* USHORT etc not supported by resize_img */

    case DT_UINT4:  
      *bscale = 1.0;
      *bzero  = ULONG_TZERO;
      return LONG_IMG;

    default:
      return SHORT_IMG;
  }
}


/***************************************************************/
/*                                                             */
/* Function:   ftp_bitpix_to_type                              */
/*                                                             */
/* Description:                                                */
/*    Figure out the data type for this image                  */
/*    The rules are different from ASCDM1.0; in particular, we */
/*    consider any nontrivial BSCALE/BZERO for an integer      */
/*    image to force double type.                              */
/*                                                             */
/*  Input parameters:                                          */
/*                long    bitpix                               */
/*                double  bscale                               */
/*                double  bzero                                */
/*                                                             */
/*  Output parameters:                                         */
/*                none                                         */
/*                                                             */
/*  Returns:                                                   */
/*                dmDataType  type                             */
/*                                                             */
/***************************************************************/
dmDataType ftp_bitpix_to_type( long bitpix, double bscale, double bzero )
{
  switch( bitpix )
  {
    case DOUBLE_IMG:
      return DT_REAL8;

    case FLOAT_IMG:   /* Could check to see if scaling requires double, but no */
      return DT_REAL4;

    case LONG_IMG:
      if ( bscale == 1.0 &&  bzero == ULONG_TZERO )
        return DT_UINT4;
      return DT_INT4;

    case SHORT_IMG:
      if ( bscale == 1.0 &&  bzero == USHORT_TZERO )
        return DT_UINT2;
      return DT_INT2;

    case BYTE_IMG:
      return DT_UINT1;

    default:
      return DT_UNK;
  }
}



/***************************************************************/
/*                                                             */
/* Function:   ftp_parse_tdim                                  */
/*                                                             */
/* Description:                                                */
/*    The TDIM convention defines multidimensional arrays      */
/*    in FITS.                                                 */
/*    e.g.     TFORM4 = '9E'                                   */
/*             TDIM4  = '(3,3)'                                */
/*                                                             */
/*  Input parameters:                                          */
/*                char   *tdim                                 */
/*                                                             */
/*  Output parameters:                                         */
/*                long  **axesp                                */
/*                long   *naxesp                               */
/*                                                             */
/*  Returns:                                                   */
/*                nothing                                      */
/*                                                             */
/***************************************************************/
void ftp_parse_tdim( char* tdim, long** axesp, long* naxesp )
{
  char **values = NULL;
  long   naxes  = 0;
  long   i      = 0;

  values = cs_parse_paren_list( tdim, &naxes );

  if ( naxes > 0 )
  {
    *axesp = calloc( naxes, UT_SZ_I );

    for ( i = 0; i < naxes; i++ )
    {
      (*axesp)[i] = atol( values[i+1] );
    }

    *naxesp = naxes;
    ar_free_cp( values, naxes+1 );
  }
  else
  {
    *axesp = NULL;
    *naxesp = 0;
  }
}



/***************************************************************/
/*                                                             */
/* Function:   ftp_recognize_spatial                           */
/*                                                             */
/* Description:                                                */
/*   Encode the special names recognized for spatial regions.  */
/*   Used in subspace parsing.                                 */
/*   This is a hack and should be replaced by a convention for */
/*   specifying region tables - maybe just that if dim=2 it's  */
/*   always a region                                           */
/*                                                             */
/*  Input parameters:                                          */
/*                char   *name                                 */
/*                                                             */
/*  Output parameters:                                         */
/*                none                                         */
/*                                                             */
/*  Returns:                                                   */
/*                int  true_or_false                           */
/*                                                             */
/***************************************************************/
int ftp_recognize_spatial( char* name )
{
  if ( cs_uceq( name, "SKY" ) || cs_uceq( name, "POS" ) || cs_uceq( name, "DET" ))
    return UT_TRUE;

  return UT_FALSE;
}

==> ciao-4.8-src-core.tar.gz: ciao-4.8/src/lib/datamodel/ftkernel/ftlib.c <==
/*                                                                
**  Copyright (C) 2002-2003,2007,2015  Smithsonian Astrophysical Observatory 
*/                                                                

/*                                                                          */
/*  This program is free software; you can redistribute it and/or modify    */
/*  it under the terms of the GNU General Public License as published by    */
/*  the Free Software Foundation; either version 3 of the License, or       */
/*  (at your option) any later version.                                     */
/*                                                                          */
/*  This program is distributed in the hope that it will be useful,         */
/*  but WITHOUT ANY WARRANTY; without even the implied warranty of          */
/*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           */
/*  GNU General Public License for more details.                            */
/*                                                                          */
/*  You should have received a copy of the GNU General Public License along */
/*  with this program; if not, write to the Free Software Foundation, Inc., */
/*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             */
/*                                                                          */

/*****************************************************************/
/*                                                               */
/*   ftlib.c                                                     */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*                                                               */
/*****************************************************************/

#include "dmilib.h"
#include "ftkernelP.h"


/***************************************************************/
/*                                                             */
/* Function:   ftp_null_keyword_compose                        */
/*                                                             */
/* Description:                                                */
/*   Generate NULL keyword appropriate for the column datatype */
/*   or "BLANK" for images.                                    */
/*                                                             */
/*   Integer types   => TNULL keyword as per FITS convention   */
/*   Char  -\                                                  */
/*   Float   | types => TDNULL keyword as per DM convention    */
/*   Double-/                                                  */
/*   Other types     => "\0", no appropriate keyword           */
/*                                                             */
/*   Write string to provided string buffer.  It is expected   */
/*   that the space is already allocated.                      */
/*                                                             */
/*   NOTE: Bad things happen in CFITSIO if you try to write    */
/*   a TNULL keyword to a non-integer column.                  */
/*                                                             */
/* Input parameters:                                           */
/*   const  dmDataType  type                                   */
/*   const  long        colno                                  */
/*          char       *buf                                    */
/*          int         is_image                               */
/*                                                             */
/*  Output parameters:                                         */
/*          char       *buf  - populated with keyword string   */
/*                                                             */
/*                                                             */
/*  Returns:                                                   */
/*    nothing                                                  */
/*                                                             */
/***************************************************************/
void ftp_null_keyword_compose( const dmDataType type, const long colno,
                               char* buf, int is_image )
{
  if ( is_image )
  {
    cs_copy( "BLANK", buf );
    return;
  }

  switch( type )
  {
    case DT_INT4:
    case DT_INT2:
    case DT_UINT2:
    case DT_UINT4:
    case DT_UINT1:
    case DT_BIT:
      sprintf( buf, "TNULL%ld", colno );
      break;
    case DT_CHAR:
    case DT_REAL4:
    case DT_REAL8:
      sprintf( buf, "TDNULL%ld", colno );
      break;
    default: 
      buf[0] = '\0';
      break;
  }
}


/***************************************************************/
/*                                                             */
/* Function:   ftp_block_type                                  */
/*                                                             */
/* Description:                                                */
/*            Convert FITSIO block type to ASCDM block type    */
/*                                                             */
/*  Input parameters:                                          */
/*                    int    ft_hdutype                        */
/*                                                             */
/*  Output parameters:                                         */
/*                    none                                     */
/*                                                             */
/*  Returns:                                                   */
/*                 dmBlockType  type                           */
/*                                                             */
/***************************************************************/
dmBlockType ftp_block_type( int ft_hdutype )
{
  switch( ft_hdutype )
  {
    case IMAGE_HDU:
      return dmIMAGE;

    case ASCII_TBL:
      return dmTABLE;

    case BINARY_TBL:
      return dmTABLE;

    default:
      break;
  }

  return dmUNKNOWNBLOCK;
}



/***************************************************************/
/*                                                             */
/* Function:   ftp_hdutype_name                                */
/*                                                             */
/* Description:                                                */
/*            Map CFITSIO HDU types to names                   */
/*                                                             */
/*  Input parameters:                                          */
/*               const int   type                              */
/*                     char *name                              */
/*                                                             */
/*  Output parameters:                                         */
/*                    none                                     */
/*                                                             */
/*  Returns:                                                   */
/*                 nothing                                     */
/*                                                             */
/***************************************************************/
void ftp_hdutype_name( const int type, char* name )
{
  char *names[] = { "IMAGE", "TABLE", "BINTABLE" };

  if ( type >= 0 && type <= 2 )
    strcpy( name, names[type] );
  else
    strcpy( name, "UNKNOWN" );
}



/***************************************************************/
/*                                                             */
/* Function:   ftp_bitpix                                      */
/*                                                             */
/* Description:                                                */
/*                                                             */
/*                                                             */
/*  Input parameters:                                          */
/*                 dmDataType  type                            */
/*                                                             */
/*  Output parameters:                                         */
/*                 double     *bscale                          */
/*                 double     *bzero                           */
/*                                                             */
/*  Returns:                                                   */
/*                 int  bitpix                                 */
/*                                                             */
/***************************************************************/
int ftp_bitpix( dmDataType type, double* bscale, double* bzero )
{
  *bscale = 1.0;
  *bzero  = 0.0;

  switch( type )
  {
    case DT_REAL8:
      return DOUBLE_IMG;

    case DT_REAL4:
      return FLOAT_IMG;

    case DT_INT2:
      return SHORT_IMG;

    case DT_INT4:
      return LONG_IMG;

    case DT_UINT1:
      return BYTE_IMG;

    case DT_UINT2: 
      *bscale = 1.0;
      *bzero  = USHORT_TZERO;
      return SHORT_IMG;  /* USHORT etc not supported by resize_img */

    case DT_UINT4:  
      *bscale = 1.0;
      *bzero  = ULONG_TZERO;
      return LONG_IMG;

    default:
      return SHORT_IMG;
  }
}


/***************************************************************/
/*                                                             */
/* Function:   ftp_bitpix_to_type                              */
/*                                                             */
/* Description:                                                */
/*    Figure out the data type for this image                  */
/*    The rules are different from ASCDM1.0; in particular, we */
/*    consider any nontrivial BSCALE/BZERO for an integer      */
/*    image to force double type.                              */
/*                                                             */
/*  Input parameters:                                          */
/*                long    bitpix                               */
/*                double  bscale                               */
/*                double  bzero                                */
/*                                                             */
/*  Output parameters:                                         */
/*                none                                         */
/*                                                             */
/*  Returns:                                                   */
/*                dmDataType  type                             */
/*                                                             */
/***************************************************************/
dmDataType ftp_bitpix_to_type( long bitpix, double bscale, double bzero )
{
  switch( bitpix )
  {
    case DOUBLE_IMG:
      return DT_REAL8;

    case FLOAT_IMG:   /* Could check to see if scaling requires double, but no */
      return DT_REAL4;

    case LONG_IMG:
      if ( bscale == 1.0 &&  bzero == ULONG_TZERO )
        return DT_UINT4;
      return DT_INT4;

    case SHORT_IMG:
      if ( bscale == 1.0 &&  bzero == USHORT_TZERO )
        return DT_UINT2;
      return DT_INT2;

    case BYTE_IMG:
      return DT_UINT1;

    default:
      return DT_UNK;
  }
}



/***************************************************************/
/*                                                             */
/* Function:   ftp_parse_tdim                                  */
/*                                                             */
/* Description:                                                */
/*    The TDIM convention defines multidimensional arrays      */
/*    in FITS.                                                 */
/*    e.g.     TFORM4 = '9E'                                   */
/*             TDIM4  = '(3,3)'                                */
/*                                                             */
/*  Input parameters:                                          */
/*                char   *tdim                                 */
/*                                                             */
/*  Output parameters:                                         */
/*                long  **axesp                                */
/*                long   *naxesp                               */
/*                                                             */
/*  Returns:                                                   */
/*                nothing                                      */
/*                                                             */
/***************************************************************/
void ftp_parse_tdim( char* tdim, long** axesp, long* naxesp )
{
  char **values = NULL;
  long   naxes  = 0;
  long   i      = 0;

  values = cs_parse_paren_list( tdim, &naxes );

  if ( naxes > 0 )
  {
    *axesp = calloc( naxes, UT_SZ_I );

    for ( i = 0; i < naxes; i++ )
    {
      (*axesp)[i] = atol( values[i+1] );
    }

    *naxesp = naxes;
    ar_free_cp( values, naxes+1 );
  }
  else
  {
    *axesp = NULL;
    *naxesp = 0;
  }
}



/***************************************************************/
/*                                                             */
/* Function:   ftp_recognize_spatial                           */
/*                                                             */
/* Description:                                                */
/*   Encode the special names recognized for spatial regions.  */
/*   Used in subspace parsing.                                 */
/*   This is a hack and should be replaced by a convention for */
/*   specifying region tables - maybe just that if dim=2 it's  */
/*   always a region                                           */
/*                                                             */
/*  Input parameters:                                          */
/*                char   *name                                 */
/*                                                             */
/*  Output parameters:                                         */
/*                none                                         */
/*                                                             */
/*  Returns:                                                   */
/*                int  true_or_false                           */
/*                                                             */
/***************************************************************/
int ftp_recognize_spatial( char* name )
{
  if ( cs_uceq( name, "SKY" ) || cs_uceq( name, "POS" ) || cs_uceq( name, "DET" ))
    return UT_TRUE;

  return UT_FALSE;
}
